LIBRARY ieee ;
USE ieee.std_logic_1164.all ;

ENTITY Transmitter_datapath IS  

PORT (
		CLK 		: 	IN STD_LOGIC;
		CD 			: 	IN STD_LOGIC;
		CE 			: 	IN STD_LOGIC;
		LDD			: 	IN STD_LOGIC;
		TX_IN		: 	IN STD_LOGIC_VECTOR (7 DOWNTO 0);
		S			:  	IN STD_LOGIC_VECTOR (1 DOWNTO 0);		
		F			: 	IN STD_LOGIC_VECTOR (1 DOWNTO 0);
		TC8 		: 	OUT STD_LOGIC;
		Serial_out 	: 	OUT STD_LOGIC
		);
		
END Transmitter_datapath ; 


ARCHITECTURE schematic OF Transmitter_datapath IS
    
COMPONENT  MUX4 IS  	
	PORT (
		Ch3, Ch2, Ch1, Ch0	: 	IN STD_LOGIC;
      E                  : 	IN STD_LOGIC;
		S 					: 	IN STD_LOGIC_VECTOR (1 DOWNTO 0);
		Y					: 	OUT STD_LOGIC
	);	
	END COMPONENT;
	
COMPONENT parity_generator_8bit IS
	PORT (
		X_in   	: 	IN STD_LOGIC_VECTOR(7 downto 0);
		EPB   	: 	OUT STD_LOGIC 
	);
	END COMPONENT;

COMPONENT shift_register_10bit IS
      Port (	
		CLK		: 	IN	STD_LOGIC;
		CD 	  	: 	IN	STD_LOGIC;
		S 	  	: 	IN	STD_LOGIC_VECTOR(1 DOWNTO 0);
		RSI 	: 	IN	STD_LOGIC;
		LSI	  	: 	IN	STD_LOGIC;
		Q 	  	: 	OUT	STD_LOGIC_VECTOR(9 DOWNTO 0);
		D_IN  	: 	IN STD_LOGIC_VECTOR(9 DOWNTO 0)
	);
	END COMPONENT ;


COMPONENT data_register_8bits IS
     PORT (	
     	CLK		: 	IN	STD_LOGIC;
		CD 	  	: 	IN	STD_LOGIC;
	    LD 	  	: 	IN	STD_LOGIC;
		Q 	  	: 	OUT	STD_LOGIC_VECTOR(7 DOWNTO 0);
		D_in	: 	IN STD_LOGIC_VECTOR(7 DOWNTO 0)
	);
	END COMPONENT ;

COMPONENT counter_mod8 IS  
	PORT(
    	CD,CLK,CE	:   IN  std_logic;
         TC8		:   OUT std_logic
          );
	END COMPONENT ;	
	
	
-- Signals
   SIGNAL	B			: STD_LOGIC_VECTOR (7 DOWNTO 0);
   SIGNAL 	Q 			: STD_LOGIC_VECTOR (9 DOWNTO 0);
   SIGNAL 	EPB, RSO  	: STD_LOGIC;

BEGIN    
	
Chip1	: 	data_register_8bits
   	    PORT MAP (
	   		LD		=> LDD,
			D_in    => TX_IN,
            CD      => CD,
            CLK     => CLK,	
            Q       => B			
		   );
		   
Chip2	: 	parity_generator_8bit
   	    PORT MAP (
	 	-- from component name 	=> to signal or port name
	   		X_in	=> B,
			EPB		=> EPB					  
		   );

Chip3	:  counter_mod8
   	    PORT MAP (
	 	-- from component name 	=> to signal or port name
	   		CD		=> CD,
			CLK		=> CLK,
			CE 		=> CE,
			TC8   	=> TC8					  
		   );
		   
Chip4	: 	shift_register_10bit
   	    PORT MAP (
	 	-- from component name 	=> to signal or port name
			CLK  => CLK,
			CD 	 => CD,
			S 	 => S,
			RSI  => '0',
			LSI	 => '0',
			Q(9 downto 3) => Q(9 downto 3),			
			Q(2) => RSO,				  
			Q(1 downto 0) => Q(1 downto 0),
			D_IN (9 downto 2)  => B,
			D_IN (1 downto 0) => "00"
		   );
		   
Chip5	:	MUX4
   	    PORT MAP (
	 	-- from component name 	=> to signal or port name
	   		Ch0		=> '1',
			Ch1		=> '0',
			Ch2		=> RSO,
			ch3   	=> EPB,
			E      => '1',
			S       => F,
         Y       => Serial_out			
		   );		   

END schematic ;